import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.*;

import javax.swing.JOptionPane;

import net.java.games.jogl.*;

public class PBufferTest
{
    int texture;
    int width;
    int height;
    float time;

    /**
     * Returns a free texture id
     */
    public static int genTexture(GL gl)
    {
        final int[] tmp = new int[1];
        gl.glGenTextures(1, tmp);
        return tmp[0];
    }

    /**
     * Sets up the frame and runs the main loop
     */
    public void start() throws IOException
    {
        GLCanvas canvas = GLDrawableFactory.getFactory().createGLCanvas(new GLCapabilities());

        final Frame frame = new Frame("PBufferTest");
        frame.setSize(800, 600);
        frame.setVisible(true);
        frame.add(canvas);
        frame.setResizable(false);
        frame.addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                System.exit(0);
            }
        });

        boolean pBufferSupport = canvas.canCreateOffscreenDrawable();

        if (!pBufferSupport)
        {
            JOptionPane.showMessageDialog(frame, "No PBuffer support. Aborting.", "Aww.", JOptionPane.ERROR);
            return;
        }

        GLCapabilities glCaps = new GLCapabilities();

        final GLPbuffer pbuffer = canvas.createOffscreenDrawable(glCaps, 256, 256);

        canvas.addGLEventListener(new GLEventListener()
        {
            boolean first = true;

            public void init(GLDrawable gld)
            {
                GL gl = gld.getGL();

                gl.glEnable(GL.GL_TEXTURE_2D);
                texture = genTexture(gl);
                gl.glBindTexture(GL.GL_TEXTURE_2D, texture);
                gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_REPEAT);
                gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_REPEAT);
                gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR);
                gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST);

                byte[] data = new byte[256 * 256 * 3];
                for (int i=0; i<data.length; i++) data[i] = (byte)(Math.random()*256);
                gl.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, 256, 256, 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE, data);
            }

            public void display(GLDrawable gld)
            {
                GL gl = gld.getGL();
                GLU glu = gld.getGLU();

                try
                {
//                  if (pbuffer.isInitialized()) // Always returns false on nvidia..
                    pbuffer.display();
                }
                catch (Throwable e)
                {
                    JOptionPane.showMessageDialog(frame, "Failed to render PBuffer. Aborting.", "Aww.", JOptionPane.ERROR);
                }

                render(gl, glu);
            }

            public void reshape(GLDrawable gld, int x, int y, int width, int height)
            {
                PBufferTest.this.width = width;
                PBufferTest.this.height = height;
            }

            public void displayChanged(GLDrawable gld, boolean arg1, boolean arg2)
            {
            }
        });

        pbuffer.addGLEventListener(new GLEventListener()
        {
            public void init(GLDrawable gld)
            {
            }

            public void display(GLDrawable gld)
            {
                GL gl = gld.getGL();
                GLU glu = gld.getGLU();

                offscreenRender(gl, glu);
            }

            public void reshape(GLDrawable gld, int x, int y, int width, int height)
            {
            }

            public void displayChanged(GLDrawable gld, boolean arg1, boolean arg2)
            {
            }
        });

        while (true)
        {
            time+=0.01f;
            canvas.display();
            Thread.yield();
        }
    }

    /**
     * Renders a spinning triangle, then copies the buffer data to a texture
     */
    void offscreenRender(GL gl, GLU glu)
    {
        gl.glViewport(0, 0, 256, 256);
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();
        gl.glOrtho(0, 256, 256, 0, -1.0, 1.0);
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();
        gl.glClearColor(0.5f, 0.5f, 0.5f, 0);
        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);

        gl.glPushMatrix();
        {
            gl.glTranslatef(256 / 2, 256 / 2, 0);
            gl.glRotatef(time * 90, 0, 0, 1);
            gl.glBegin(GL.GL_TRIANGLE_FAN);
            {
                gl.glColor4f(1, 0, 0, 1);
                gl.glVertex2f((float)Math.sin(0 / 3f * Math.PI * 2) * 128, (float)Math.cos(0 / 3f * Math.PI * 2) * 128);
                gl.glColor4f(0, 1, 0, 1);
                gl.glVertex2f((float)Math.sin(1 / 3f * Math.PI * 2) * 128, (float)Math.cos(1 / 3f * Math.PI * 2) * 128);
                gl.glColor4f(0, 0, 1, 1);
                gl.glVertex2f((float)Math.sin(2 / 3f * Math.PI * 2) * 128, (float)Math.cos(2 / 3f * Math.PI * 2) * 128);
            }
            gl.glEnd();
            gl.glColor3f(1, 1, 1);
        }
        gl.glPopMatrix();

        gl.glFinish(); // Don't copy the data until it the rendering is complete

        gl.glEnable(GL.GL_TEXTURE_2D);
        gl.glBindTexture(GL.GL_TEXTURE_2D, texture);
        gl.glCopyTexSubImage2D(GL.GL_TEXTURE_2D, 0, 0, 0, 0, 0, 256, 256);
        gl.glDisable(GL.GL_TEXTURE_2D);

        int error = gl.glGetError();

        if (error > 0)
        {
            System.out.println("Offscreen: OpenGL error " + error + ":" + glu.gluErrorString(error));
        }
    }

    /**
     * Renders a spinning textured quad.
     * This uses the texture generated by the offscreen render call
     */
    void render(GL gl, GLU glu)
    {
        gl.glViewport(0, 0, width, height);
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();
        gl.glOrtho(0, width, height, 0, -1.0, 1.0);
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();
        gl.glClearColor(0.75f, 0.5f, 0.25f, 0);
        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);

        gl.glPushMatrix();
        {
            gl.glTranslatef(width / 2, height / 2, 0);
            gl.glRotatef(time * 10, 0, 0, 1);
            gl.glEnable(GL.GL_TEXTURE_2D);
            gl.glBindTexture(GL.GL_TEXTURE_2D, texture);
            gl.glBegin(GL.GL_TRIANGLE_FAN);
            {
                gl.glTexCoord2f(0, 0);
                gl.glVertex2f(-width / 3, -width / 3);
                gl.glTexCoord2f(1, 0);
                gl.glVertex2f(width / 3, -width / 3);
                gl.glTexCoord2f(1, 1);
                gl.glVertex2f(width / 3, width / 3);
                gl.glTexCoord2f(0, 1);
                gl.glVertex2f(-width / 3, width / 3);
            }
            gl.glEnd();
            gl.glDisable(GL.GL_TEXTURE_2D);
        }
        gl.glPopMatrix();

        int error = gl.glGetError();

        if (error > 0)
        {
            System.out.println("Framebuffer: OpenGL error " + error + ":" + glu.gluErrorString(error));
        }
    }

    public static void main(String[] args)
    {
        try
        {
            new PBufferTest().start();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }
}